(************************************************************************)
(*		Convert "GRAPHICS.SRC" into an include file for Berzerk			*)
(*		Game for TI83/TI83+												*)
(************************************************************************)

program ConvGfx;

type PixelSet = array[0..7] of Byte;	{ A single graphic definition }

type Graphic = record   				{ Record for one group }
	Used:Boolean;						{ True if used }
	Name:String;						{ Name of the graphic group }
	Count:Integer;						{ Number of sets defined }
	Defined:array[0..3] of Boolean;		{ True if graphic defined }
	Pixels:array[0..3] of PixelSet;		{ Corresponding graphic set }
end;

const MaxGraphics = 32;					{ Maximum number of graphics sets }

var Gfx:array[1..MaxGraphics] of		{ Declare the main data structure }
							Graphic;

(************************************)
(*	Get Record ID for given name	*)
(************************************)

function GetRecordID(Name:String):Integer;
var n,ID:Integer;
begin
	while Name[1] in ['0'..'9','#'] do	{ Strip # and numbers }
							Delete(Name,1,1);
	for n := 1 to Length(Name) do		{ Make it upper case }
				Name[n] := UpCase(Name[n]);
	while Length(Name) < 14 do			{ Pad out to 14 characters }
				Name := Concat(Name,' ');
	ID := 0;							{ Not found yet }
	n  := 1;							{ Start of search }

	while ID = 0 do						{ Wait until found }
		begin
			if not Gfx[n].Used then		{ Empty slot, must be new type }
				begin
					ID := n;			{ Set it up }
					Gfx[ID].Used := True;
					Gfx[ID].Name := Name;
				end
			else						{ if matches one requested, use that }
				begin
					if Name = Gfx[n].Name then ID := n;
				end;
			Inc(n);
		end;

	GetRecordID := ID;
end;

(************************************************)
(*	Read one graphic definition from the file 	*)
(************************************************)

procedure ReadPixelSet(var f:Text;var Pix:PixelSet);
var i,j:Integer;
	s:String;
begin
	for i := 0 to 7 do					{ Read 8 lines }
		begin
			Pix[i] := 0;
			ReadLn(f,s);    			{ Read representation }
			for j := 1 to Length(s) do	{ Convert to binary }
				if s[j] = 'X' then Pix[i] := Pix[i] + (256 shr j);
		end;
end;

(****************************************)
(* 	Read in a complete set of graphics 	*)
(****************************************)

procedure LoadGraphic;
var f:Text;
	i,j:Integer;
	s:String;
	Gr:PixelSet;
begin

	for i := 1 to MaxGraphics do		{ Delete all records }
		with Gfx[i] do
			begin
				Used := False;Name := '';Count := 0;
				for j := 0 to 3 do Defined[j] := False;
			end;

	Assign(f,'GRAPHICS.SRC');			{ Open the source file }
	Reset(f);

	WriteLn(';');
	WriteLn(';  Generated by a program.');
	WriteLn(';');
	while not eof(f) do   				{ Work through file }
		begin
			ReadLn(f,s);
			if s[1] <> ';' then			{ Is it not a comment ? }
				begin
					if s[1] <> '#' then Halt(1);
					i := GetRecordID(s);
					ReadPixelSet(f,Gr);
					Delete(s,1,1);
					if not (s[1] in ['0'..'9']) then
										s := Concat('0',s);
					while s[1] in ['0'..'9'] do
						begin
							j := Ord(s[1]) - 48;
							Delete(s,1,1);
							Gfx[i].Defined[j] := True;
							Gfx[i].Pixels[j] := Gr;
							Inc(Gfx[i].Count);
						end;
				end;
		end;
	Close(f);
end;

(********************************)
(*  Write a set of pixels out   *)
(********************************)

procedure DumpPixelGroup(Pix:PixelSet);
var i:Integer;
	s:String;
begin
	s := '0123456789ABCDEF';
	Write('          .db   ');
	for i := 0 to 7 do
		begin
			Write('$',s[Pix[i] div 16+1],s[Pix[i] mod 16+1]);
			if i <> 7 then Write(',');
		end;
	WriteLn;
end;

(********************************************************)
(*	Dump all records with a count equal to that given 	*)
(********************************************************)

procedure DumpRecord(Count:Integer;var CharID:Integer);
var i,j,n:Integer;
begin
	for i := 1 to MaxGraphics do
		if Gfx[i].Used then
			if Gfx[i].Count = Count then
				begin
					n := CharID;
					if Count = 4 then n := n+$80;
					WriteLn('GR_',Gfx[i].Name,'  = ',n);
					for j := 0 to 3 do
						if Gfx[i].Defined[j] then
							begin
								DumpPixelGroup(Gfx[i].Pixels[j]);
								Inc(CharID);
							end;
					WriteLn;
				end;
end;

var CharID:Integer;

begin
	LoadGraphic;
	CharID := 0;
	DumpRecord(4,CharID);
	DumpRecord(1,CharID);
end.


